@@ -49,7 +49,7 @@ class DataSource(models.Model):
         return separator.join(abbrs[::-1])#reverse the order so it proceeds heirarchically
         
     def _hr_descriptor(self):
-        return 'Source: %s' % self._descriptor(separator=">>")
+        return '%s' % self._descriptor(separator=">>")
     hr_descriptor = property(_hr_descriptor)
     
     def _mr_descriptor(self):
@@ -64,10 +64,13 @@ class DataSource(models.Model):
     top_level_source = property(_top_level_source)
     
     def get_absolute_url(self):
-        return reverse('djhcup_staging.views.source_detail', kwargs={'source_id': self.pk})
+        return reverse(
+            'djhcup_staging.views.obj_detail',
+            kwargs={'obj_id': self.pk, 'obj_type': 'DataSource'}
+        )
     
     def __unicode__(self):
-        return self.hr_descriptor
+        return "<DataSource: {hr}>".format(hr=self.hr_descriptor)
 
 
 class FileType(models.Model):
@@ -113,10 +116,13 @@ class File(models.Model):
     loadfile = models.ForeignKey('self', blank=True, default=None, null=True)
     
     
-    def _hr_size(self):
-        """Returns size in MB"""
+    def _hr_size(self, MiB=True):
+        """Returns size in MiB or MB"""
         if self.size_on_disk:
-            return self.size_on_disk/1000000
+            if MiB:
+                return self.size_on_disk/1048576
+            else:
+                return self.size_on_disk/1000000
         else:
             return None
     hr_size = property(_hr_size)
@@ -145,23 +151,22 @@ class File(models.Model):
         
         return handle
     
-    
     class Meta:
         verbose_name = 'file'
         verbose_name_plural = 'files'
         ordering = ['file_type__source', 'state__abbreviation', 'year', 'file_type__compression', 'filename']
     
-    
     def get_absolute_url(self):
-        return reverse('djhcup_staging.views.file_detail', kwargs={'file_id': self.pk})
-    
+        return reverse(
+            'djhcup_staging.views.obj_detail',
+            kwargs={'obj_id': self.pk, 'obj_type': 'File'}
+        )
     
     def __unicode__(self):
-        return '<File: %s (%s)>' % (self.filename, self.file_type)
+        return '<File #%s: %s (%s)>' % (self.pk, self.filename, self.file_type)
 
 
 class ImportBatch(models.Model):
-    #destination = models.ForeignKey('ImportTable')
     name = models.CharField(max_length=200, blank=True, help_text="Optional name for this batch of imports")
     description = models.TextField(blank=True, help_text="Optional description for this batch of imports")
     STATUS_CHOICES = [('NEW', 'Not started'),
@@ -231,9 +236,11 @@ class ImportBatch(models.Model):
         else:
             return False
     
-    
     def get_absolute_url(self):
-        return reverse('djhcup_staging.views.importbatch_detail', kwargs={'batch_id': self.pk})
+        return reverse(
+            'djhcup_staging.views.obj_detail',
+            kwargs={'obj_id': self.pk, 'obj_type': 'ImportBatch'}
+        )
     
     def __unicode__(self):
         c_items = ImportQueue.objects.filter(batch=self).count()
@@ -438,10 +445,9 @@ class ImportQueue(models.Model):
         
         return meta
     
-    
     def __unicode__(self):
         return "ImportQueue item %s [%s]: %s" % (self.pk, self.status, self.file.filename)
-
+    
 
 class StagingBatch(models.Model):
     #destination = models.ForeignKey('ImportTable')
@@ -537,9 +543,11 @@ class StagingBatch(models.Model):
         logger.log(level, logger_message)
         return True
     
-    
     def get_absolute_url(self):
-        return reverse('djhcup_staging.views.stagingbatch_detail', kwargs={'batch_id': self.pk})
+        return reverse(
+            'djhcup_staging.views.obj_detail',
+            kwargs={'obj_id': self.pk, 'obj_type': 'StagingBatch'}
+        )
     
     def __unicode__(self):
         c_items = StagingQueue.objects.filter(batch=self).count()
@@ -724,7 +732,7 @@ class StagingQueue(models.Model):
     
     def __unicode__(self):
         return "StagingQueue item %s [%s]" % (self.pk, self.status)
-
+    
 
 class StagingTable(models.Model):
     """This is a record of staging tables created/populated
@@ -751,10 +759,7 @@ class StagingTable(models.Model):
     ]
     category = models.CharField(max_length=100, choices=STG_CAT_CHOICES, blank=False)
     wip = models.BooleanField(default=True, help_text="This StagingTable is work-in-progress, and should not be used.")
-    
     records = models.IntegerField()
-    #TODO: consider adding a col count
-    #col_count = models.IntegerField(blank=False)
     
     #meta cruft
     created_at = models.DateTimeField(auto_now_add=True)
@@ -766,18 +771,38 @@ class StagingTable(models.Model):
     # methods/properties for determining state and year
     def _state_abbr(self):
         if self.parent is None:
-            return self.imported_via.file.state.abbreviation
+            if self.imported_via is not None:
+                return self.imported_via.file.state.abbreviation
+            else:
+                return None
         else:
             return self.parent._state_abbr()
     state = property(_state_abbr)
         
     def _year(self):
         if self.parent is None:
-            return self.imported_via.file.year
+            if self.imported_via is not None:
+                return self.imported_via.file.year
+            else:
+                return None
         else:
             return self.parent.year
     year = property(_year)
     
+    def _data_source(self):
+        if self.parent is None:
+            if self.imported_via is not None:
+                return self.imported_via.file.file_type.source
+            else:
+                return None
+        else:
+            return self.parent.data_source
+    data_source = property(_data_source)
+    
+    def _col_count(self):
+        return self.column_set.count()
+    col_count = property(_col_count)
+    
     def __unicode__(self):
         return "StagingTable %d: %s" % (self.pk, self.name)
     
@@ -802,6 +827,12 @@ class StagingTable(models.Model):
                 default_logger.error('Unable to drop table {table} from database'.format(table=self.name))
         
         super(StagingTable, self).delete(*args, **kwargs)
+    
+    def get_absolute_url(self):
+        return reverse(
+            'djhcup_staging.views.obj_detail',
+            kwargs={'obj_id': self.pk, 'obj_type': 'StagingTable'}
+        )
 
 
 class Column(models.Model):
@@ -852,7 +883,6 @@ class Column(models.Model):
                                       blank=True, default=None, null=True)
     mean = models.DecimalField(max_digits=50, decimal_places=6,
                                blank=True, default=None, null=True)
-    
     ref_col = models.ForeignKey('ReferenceColumn', blank=True, default=None,
                                 null=True)
     
@@ -891,7 +921,6 @@ class Column(models.Model):
             logger.warning("match_ref_col() called on Column %s with ref_col already set to ReferenceColumn %s. match_ref_col() will not proceed." % (self.pk, self.ref_col.pk))
             return None
     
-    
     def validate(self, tolerance=0.01):
         """
         Attempts to validate the integrity of data in this Column by
@@ -971,6 +1000,12 @@ class Column(models.Model):
                     self.integrity_status = 'UNAVAILABLE_AUTO'
                     self.save()
                     return None
+    
+    def get_absolute_url(self):
+        return reverse(
+            'djhcup_staging.views.obj_detail',
+            kwargs={'obj_id': self.pk, 'obj_type': 'Column'}
+        )
 
 
 class ReferenceColumn(models.Model):
@@ -992,3 +1027,9 @@ class ReferenceColumn(models.Model):
     
     def __unicode__(self):
         return "<ReferenceColumn %s: %s %s %s %s>" % (self.pk, self.source, self.state.abbreviation, self.year, self.name)
+    
+    def get_absolute_url(self):
+        return reverse(
+            'djhcup_staging.views.obj_detail',
+            kwargs={'obj_id': self.pk, 'obj_type': 'ReferenceColumn'}
+        )
